@@ -52,23 +52,34 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl-lib))
-
 (require 'chess-message)
 (require 'chess-ply)
 (require 'chess-pos)
+(require 'cl-lib)
+
+(defconst chess-algebraic-figurine-pieces
+  '((?K . #x2654) (?Q . #x2655) (?R . #x2656)
+    (?B . #x2657) (?N . #x2658) (?P . #x2659)
+    (?k . #x265A) (?q . #x265B) (?r . #x265C)
+    (?b . #x265D) (?n . #x265E) (?p . #x265F))
+  "Map internal piece representation to Unicode chess figures (as used in figurine
+notation.")
 
 (defconst chess-algebraic-regexp
-  (rx (group (or (or "O-O" "O-O-O")
-		 (and (optional (group (char ?N ?B ?R ?Q ?K)))
+  (rx (group (or (or "O-O" "O-O-O" "0-0" "0-0-0")
+		 (and (optional (group (char ?N ?B ?R ?Q ?K
+					     ?♔ ?♕ ?♖ ?♗ ?♘
+					     ?♚ ?♛ ?♜ ?♝ ?♞)))
 		      (optional (char ?/))
 		      (group (optional (char "a-h")) (optional (char "1-8")))
 		      (optional (group (char ?- ?x)))
 		      (group (char "a-h") (char "1-8"))
-		      (optional (group ?= (group (char ?N ?B ?R ?Q ?K)))))))
+		      (optional (group ?= (group (char ?N ?B ?R ?Q ?K
+						       ?♔ ?♕ ?♖ ?♗ ?♘
+						       ?♚ ?♛ ?♜ ?♝ ?♞)))))))
       (optional (group (char ?+ ?#))))
   "A regular expression that matches all possible algebraic moves.
-This regexp handles both long and short form.")
+This regexp matches short, long and figurine notation.")
 
 (defconst chess-algebraic-regexp-entire (concat chess-algebraic-regexp "$"))
 
@@ -82,16 +93,16 @@ This regexp handles both long and short form.")
     (at-move-string    . "At algebraic move '%s': %s")))
 
 (defun chess-algebraic-to-ply (position move &optional trust)
-  "Convert the algebraic notation MOVE for POSITION to a ply."
-  (cl-assert (vectorp position))
-  (cl-assert (stringp move))
+  "Convert (short, long or figurine) algebraic notation MOVE for POSITION to a ply."
+  (cl-check-type position chess-pos)
+  (cl-check-type move string)
   (let ((case-fold-search nil))
     (when (string-match chess-algebraic-regexp-entire move)
       (let ((color (chess-pos-side-to-move position))
 	    (mate (match-string 8 move))
 	    (piece (aref move 0))
-	    changes long-style)
-	(if (eq piece ?O)
+	    changes type)
+	(if (or (eq piece ?O) (eq piece ?0))
 	    (setq changes (chess-ply-castling-changes
 			   position (= (length (match-string 1 move)) 5)))
 	  (let ((promotion (match-string 7 move)))
@@ -102,11 +113,17 @@ This regexp handles both long and short form.")
 	       (if (and source (= (length source) 2))
 		   (prog1
 		       (list (chess-coord-to-index source) target)
-		     (setq long-style t))
+		     (setq type :lan))
 		 (if (= (length source) 0)
 		     (setq source nil)
 		   (setq source (aref source 0)))
 		 (let (candidates which)
+		   (when (and (not type) (< piece ?a))
+		     (setq type :san))
+		   (when (rassq piece chess-algebraic-figurine-pieces)
+		     (unless type (setq type :fan))
+		     (setq piece (upcase
+				  (car (rassq piece chess-algebraic-figurine-pieces)))))
 		   (unless (< piece ?a)
 		     (setq source piece piece ?P))
 		   ;; we must use our knowledge of how pieces can
@@ -135,16 +152,24 @@ This regexp handles both long and short form.")
 			       (chess-error 'could-not-clarify)
 			     (list which target))))
 		     (chess-error 'no-candidates move))))))
+
 	    (when promotion
-	      (nconc changes (list :promote (aref promotion 0))))))
+	      (nconc changes
+		     (list :promote
+			   (upcase (or (car (rassq (aref promotion 0)
+						   chess-algebraic-figurine-pieces))
+				       (aref promotion 0))))))))
 
 	(when changes
 	  (if (and trust mate)
-	      (nconc changes (list (if (equal mate "#")
+	      (nconc changes (list (if (string-equal mate "#")
 				       :checkmate
 				     :check))))
-	  (unless long-style
-	    (nconc changes (list :san move)))
+	  ;; If we know the notation type by now, remember the string so that
+	  ;; we do not need to re-generate it later on.
+	  (when type
+	    (cl-check-type type keyword)
+	    (nconc changes (list type move)))
 
 	  (condition-case err
 	      (apply 'chess-ply-create position trust changes)
@@ -152,61 +177,83 @@ This regexp handles both long and short form.")
 	     (chess-error 'at-move-string
 			  move (error-message-string err)))))))))
 
-(defun chess-ply--move-text (ply long)
-  (or
-   (and (chess-ply-keyword ply :castle) "O-O")
-   (and (chess-ply-keyword ply :long-castle) "O-O-O")
-   (let* ((pos (chess-ply-pos ply))
-	  (from (chess-ply-source ply))
-	  (to (chess-ply-target ply))
-	  (from-piece (chess-pos-piece pos from))
-	  (rank 0) (file 0)
-	  (from-rank (chess-index-rank from))
-	  (from-file (chess-index-file from))
-	  (differentiator (chess-ply-keyword ply :which)))
-	(unless differentiator
-	  (let ((candidates (chess-search-position pos to from-piece nil t)))
-	    (when (> (length candidates) 1)
-	      (dolist (candidate candidates)
-		(when (= (chess-index-rank candidate) from-rank)
-		  (setq rank (1+ rank)))
-		(when (= (chess-index-file candidate) from-file)
-		  (setq file (1+ file))))
-	      (cond
-	       ((= file 1)
-		(setq differentiator (+ from-file ?a)))
-	       ((= rank 1)
-		(setq differentiator (+ (- 7 from-rank) ?1)))
-	       (t (chess-error 'could-not-diff)))
-	      (chess-ply-set-keyword ply :which differentiator))))
-	(concat
-	 (unless (= (upcase from-piece) ?P)
-	   (char-to-string (upcase from-piece)))
-	 (cond
-	  (long (chess-index-to-coord from))
-	  (differentiator (char-to-string differentiator))
-	  ((and (not long) (= (upcase from-piece) ?P)
-		(/= from-file (chess-index-file to)))
-	   (char-to-string (+ from-file ?a))))
-	 (if (or (/= ?  (chess-pos-piece pos to))
-		 (chess-ply-keyword ply :en-passant))
-	     "x" (if long "-"))
-	 (chess-index-to-coord to)
-	 (let ((promote (chess-ply-keyword ply :promote)))
-	   (if promote
-	       (concat "=" (char-to-string promote))))
-	 (if (chess-ply-keyword ply :check) "+"
-	   (if (chess-ply-keyword ply :checkmate) "#"))))))
-
-(defun chess-ply-to-algebraic (ply &optional long)
+(defun chess-ply-to-algebraic (ply &optional type)
   "Convert the given PLY to algebraic notation.
-If LONG is non-nil, render the move into long notation."
-  (cl-assert (listp ply))
-  (or (and (not long) (chess-ply-keyword ply :san))
+Optional argument TYPE specifies the kind of algebraic notation to generate.
+`:san' (the default) generates short (or standard) algebraic notation
+\(like \"Nc3\").  `:lan' generates long algebraic notation (like \"Nb1-c3\".
+`:fan' generates figurine algebraic notation (like \"♘c3\".
+Finally, `:numeric' generates ICCF numeric notation (like \"2133\"."
+  (cl-check-type ply (and list (not null)))
+  (cl-check-type type (member nil :san :fan :lan :numeric))
+  (unless type (setq type :san))
+  (or (chess-ply-keyword ply type)
       (and (null (chess-ply-source ply)) "")
-      (let ((move (chess-ply--move-text ply long)))
-	(unless long (chess-ply-set-keyword ply :san move))
-	move)))
+      (chess-ply-set-keyword
+       ply type
+       (or
+	(and (eq type :numeric)
+	     (apply
+	      #'string
+	      (+ (chess-index-file (chess-ply-source ply)) ?1)
+	      (+ (chess-index-rank (logxor (chess-ply-source ply) #o70)) ?1)
+	      (+ (chess-index-file (chess-ply-target ply)) ?1)
+	      (+ (chess-index-rank (logxor (chess-ply-target ply) #o70)) ?1)
+	      (when (chess-ply-keyword ply :promote)
+		(list (+ (cl-position (chess-ply-keyword ply :promote)
+				      '(?Q ?R ?B ?N)) ?1)))))
+	(and (chess-ply-keyword ply :castle) "O-O")
+	(and (chess-ply-keyword ply :long-castle) "O-O-O")
+	(let* ((pos (chess-ply-pos ply))
+	       (from (chess-ply-source ply))
+	       (to (chess-ply-target ply))
+	       (from-piece (chess-pos-piece pos from))
+	       (rank 0) (file 0)
+	       (from-rank (chess-index-rank from))
+	       (from-file (chess-index-file from))
+	       (differentiator (chess-ply-keyword ply :which)))
+	  (unless differentiator
+	    (let ((candidates (chess-search-position pos to from-piece nil t)))
+	      (when (> (length candidates) 1)
+		(dolist (candidate candidates)
+		  (when (= (chess-index-rank candidate) from-rank)
+		    (setq rank (1+ rank)))
+		  (when (= (chess-index-file candidate) from-file)
+		    (setq file (1+ file))))
+		(cond
+		 ((= file 1)
+		  (setq differentiator (+ from-file ?a)))
+		 ((= rank 1)
+		  (setq differentiator (+ (- 7 from-rank) ?1)))
+		 (t (chess-error 'could-not-diff)))
+		(chess-ply-set-keyword ply :which differentiator))))
+	  (concat
+	   (unless (= (upcase from-piece) ?P)
+	     (char-to-string
+	      (cond ((memq type '(:san :lan)) (upcase from-piece))
+		    ((eq type :fan)
+		     (cdr (assq from-piece chess-algebraic-figurine-pieces))))))
+	   (cond
+	    ((eq type :lan) (chess-index-to-coord from))
+	    (differentiator (char-to-string differentiator))
+	    ((and (not (eq type :lan)) (= (upcase from-piece) ?P)
+		  (/= from-file (chess-index-file to)))
+	     (char-to-string (+ from-file ?a))))
+	   (if (or (/= ?  (chess-pos-piece pos to))
+		   (chess-ply-keyword ply :en-passant))
+	       "x" (if (eq type :lan) "-"))
+	   (chess-index-to-coord to)
+	   (let ((promote (chess-ply-keyword ply :promote)))
+	     (if promote
+		 (concat "=" (char-to-string
+			      (cond ((eq type :fan)
+				     (cdr (assq (if (chess-pos-side-to-move pos)
+						    promote
+						  (downcase promote))
+						chess-algebraic-figurine-pieces)))
+				    (t promote))))))
+	   (if (chess-ply-keyword ply :check) "+"
+	     (if (chess-ply-keyword ply :checkmate) "#"))))))))
 
 (provide 'chess-algebraic)
 
