@@ -25,15 +25,18 @@
 
 ;;; Code:
 
+(require 'chess-fen)
+(require 'chess-input)
 (require 'chess-message)
 (require 'chess-module)
-(require 'chess-var)
-(require 'chess-input)
+(require 'chess-pgn)
 (require 'chess-random)
+(require 'chess-var)
 
 (defgroup chess-display nil
-  "Common code used by chess displays."
-  :group 'chess)
+  "Options common to all chessboard displays."
+  :group 'chess
+  :link '(custom-manual "(chess)Chessboard displays"))
 
 (defcustom chess-display-popup t
   "If non-nil (the default), popup displays whenever a significant event
@@ -43,7 +46,7 @@ occurs."
 
 (make-variable-buffer-local 'chess-display-popup)
 
-(defcustom chess-display-highlight-legal nil
+(defcustom chess-display-highlight-legal t
   "If non-nil, highlight legal target squares when a piece is selected."
   :type 'boolean
   :group 'chess-display)
@@ -349,7 +352,19 @@ also view the same game."
 		  (point-min))))))
     (aref chess-display-index-positions index)))
 
+(defun chess-display-draw-square (display index piece &optional pos)
+  (cl-check-type display (or null buffer))
+  (cl-check-type index (integer 0 63))
+  (cl-check-type piece (member nil ?  ?P ?N ?B ?R ?Q ?K ?p ?n ?b ?r ?q ?k))
+  (chess-with-current-buffer display
+    (cl-check-type pos (or null (number ((point-min)) ((point-max)))))
+    (funcall chess-display-event-handler 'draw-square
+	     (or pos (chess-display-index-pos nil index))
+	     (or piece (chess-pos-piece (chess-display-position nil) index))
+	     index)))
+
 (defun chess-display-paint-move (display ply)
+  (cl-check-type display (or null buffer))
   (chess-with-current-buffer display
     (if chess-display-highlight-last-move
 	(chess-display-redraw))
@@ -360,22 +375,15 @@ also view the same game."
 	    (setq ch nil)
 	  (let ((from (car ch))
 		(to (cadr ch)))
-	    (funcall chess-display-event-handler 'draw-square
-		     (chess-display-index-pos nil from) ?  from)
-	    (let ((new-piece (chess-ply-keyword ply :promote)))
-	      (if new-piece
-		  (funcall chess-display-event-handler 'draw-square
-			   (chess-display-index-pos nil to)
-			   (if (chess-pos-side-to-move position)
-			       new-piece
-			     (downcase new-piece)) to)
-		(funcall chess-display-event-handler 'draw-square
-			 (chess-display-index-pos nil to)
-			 (chess-pos-piece position from) to)))
+	    (chess-display-draw-square nil from ? )
+	    (chess-display-draw-square
+	     nil to (or (let ((new-piece (chess-ply-keyword ply :promote)))
+			  (when new-piece
+			    (if (chess-pos-side-to-move position)
+				new-piece (downcase new-piece))))
+			(chess-pos-piece position from)))
 	    (when (chess-ply-keyword ply :en-passant)
-	      (funcall chess-display-event-handler 'draw-square
-		       (chess-display-index-pos nil (chess-pos-en-passant position))
-		       ?  (chess-pos-en-passant position))))
+	      (chess-display-draw-square nil (chess-pos-en-passant position) ? )))
 	  (setq ch (cddr ch)))))
     (if chess-display-highlight-last-move
 	(chess-display-highlight-move display ply))))
@@ -393,10 +401,9 @@ Basically, it means we are playing, not editing or reviewing."
        (not (chess-game-over-p chess-module-game))
        (not chess-display-edit-mode)))
 
-(defun chess-display-move (display ply &optional _prev-pos _pos)
+(defun chess-display-move (display ply)
   "Move a piece on DISPLAY, by applying the given PLY.
-The position of PLY must match the currently displayed position.
-If only START is given, it must be in algebraic move notation."
+The position of PLY must match the currently displayed position."
   (chess-with-current-buffer display
     (if (and (chess-display-active-p)
 	     ;; `active' means we're playing against an engine
@@ -413,6 +420,7 @@ If only START is given, it must be in algebraic move notation."
 	  (chess-game-move chess-module-game ply)
 	  (chess-display-paint-move nil ply)
 	  (chess-display-set-index* nil (chess-game-index chess-module-game))
+	  (redisplay)                   ; FIXME: This is clearly necessary, but why?
 	  (chess-game-run-hooks chess-module-game 'post-move))
       ;; jww (2002-03-28): This should beget a variation within the
       ;; game, or alter the game, just as SCID allows
@@ -430,20 +438,18 @@ that is supported by most displays, and is the default mode."
 	    (setq mode arg)
 	  (funcall chess-display-event-handler 'highlight arg mode))))))
 
-(defun chess-display-highlight-legal (display pos)
-  "Highlight all legal move targets from POS."
+(defun chess-display-highlight-legal (display index)
+  "Highlight all legal move targets from INDEX."
   (chess-with-current-buffer display
-    (dolist (ply (chess-legal-plies (chess-display-position nil)
-				    :index pos))
+    (dolist (ply (chess-legal-plies (chess-display-position nil) :index index))
       (chess-display-highlight nil "pale green"
 			       (chess-ply-target ply)))))
 
 (defun chess-display-highlight-move (display ply)
   "Highlight the last move made in the current game."
-  (chess-with-current-buffer display
-     (chess-display-highlight nil "medium sea green"
-			      (chess-ply-source ply)
-			      (chess-ply-target ply))))
+  (chess-display-highlight display "medium sea green"
+			   (chess-ply-source ply)
+			   (chess-ply-target ply)))
 
 (defun chess-display-highlight-passed-pawns (&optional display)
   (interactive)
@@ -638,6 +644,7 @@ See `chess-display-type' for the different kinds of displays."
 		   ?o ?O ?x ?=))
       (define-key map (vector key) 'chess-input-shortcut))
     (define-key map [backspace] 'chess-input-shortcut-delete)
+    (define-key map "\d" 'chess-input-shortcut-delete)
 
     (define-key map [(control ?m)] 'chess-display-select-piece)
     (define-key map [return] 'chess-display-select-piece)
@@ -1119,12 +1126,12 @@ to the end or beginning."
 (defun chess-display-set-piece (&optional piece)
   "Set the piece under point to command character, or space for clear."
   (interactive)
-  (if (or (null piece) (characterp piece))
-      (let ((index (get-text-property (point) 'chess-coord)))
-	(chess-pos-set-piece chess-display-edit-position index
-			     (or piece last-command-event))
-	(funcall chess-display-event-handler 'draw-square
-		 (point) (or piece last-command-event) index))))
+  (when (or (null piece) (characterp piece))
+    (let ((index (get-text-property (point) 'chess-coord)))
+      (chess-pos-set-piece chess-display-edit-position index
+			   (or piece last-command-event))
+      (chess-display-draw-square nil index
+				 (or piece last-command-event) (point)))))
 
 (unless (fboundp 'event-window)
   (defalias 'event-point 'ignore))
@@ -1185,8 +1192,7 @@ Clicking once on a piece selects it; then click on the target location."
 	      (if chess-display-last-selected
 		  (let ((last-sel chess-display-last-selected))
 		    ;; if they select the same square again, just deselect
-		    ;; it by redrawing the display and removing all
-		    ;; highlights
+		    ;; it by redrawing the square to remove highlights.
 		    (if (= (point) (car last-sel))
 			(funcall chess-display-event-handler 'draw-square
 				 (car last-sel)
@@ -1210,10 +1216,17 @@ Clicking once on a piece selects it; then click on the target location."
 							      coord))
 			    (throw 'message (chess-string 'move-not-legal)))
 			  (condition-case err
-			      (chess-display-move nil ply
-						  (car last-sel) (point))
+			      (chess-display-move nil ply)
 			    (error
 			     (throw 'message (error-message-string err)))))))
+		    ;; Redraw legal targets to clear highlight.
+		    (when chess-display-highlight-legal
+		      (dolist (index (mapcar #'chess-ply-target
+					     (chess-legal-plies
+					      position
+					      :index (cdr last-sel))))
+			(unless (= index coord)
+			  (chess-display-draw-square nil index))))
 		    (setq chess-display-last-selected nil))
 		(let ((piece (chess-pos-piece position coord)))
 		  (cond
