@@ -78,6 +78,8 @@
   (cl-assert (vectorp position))
   (setcar ply position))
 
+(gv-define-simple-setter chess-ply-pos chess-ply-set-pos)
+
 (defsubst chess-ply-changes (ply)
   (cl-assert (listp ply))
   (cdr ply))
@@ -87,7 +89,11 @@
   (cl-assert (listp changes))
   (setcdr ply changes))
 
+(gv-define-simple-setter chess-ply-changes chess-ply-set-changes)
+
 (defun chess-ply-any-keyword (ply &rest keywords)
+  "Return non-nil if PLY contains at least one of KEYWORDS."
+  (declare (side-effect-free t))
   (cl-assert (listp ply))
   (catch 'found
     (dolist (keyword keywords)
@@ -95,13 +101,11 @@
 	  (throw 'found keyword)))))
 
 (defun chess-ply-keyword (ply keyword)
+  (declare (side-effect-free t))
   (cl-assert (listp ply))
   (cl-assert (symbolp keyword))
   (let ((item (memq keyword (chess-ply-changes ply))))
-    (if item
-	(if (eq item (last (chess-ply-changes ply)))
-	    t
-	  (cadr item)))))
+    (and item (if (not (cdr item)) t (cadr item)))))
 
 (defun chess-ply-set-keyword (ply keyword &optional value)
   (cl-assert (listp ply))
@@ -109,13 +113,15 @@
   (let* ((changes (chess-ply-changes ply))
 	 (item (memq keyword changes)))
     (if item
-	(if value
-	    (setcar (cdr item) value))
+	(when value
+	  (setcar (cdr item) value))
       (nconc changes (if value
 			 (list keyword value)
 		       (list keyword))))
     value))
 
+(gv-define-simple-setter chess-ply-keyword chess-ply-set-keyword)
+
 (defsubst chess-ply-source (ply)
   "Returns the source square index value of PLY."
   (cl-assert (listp ply))
@@ -131,6 +137,7 @@
 	 (cadr changes))))
 
 (defsubst chess-ply-next-pos (ply)
+  "Return the position that results from executing PLY."
   (cl-assert (listp ply))
   (or (chess-ply-keyword ply :next-pos)
       (let ((position (apply 'chess-pos-move
@@ -139,12 +146,6 @@
 	(chess-pos-set-preceding-ply position ply)
 	(chess-ply-set-keyword ply :next-pos position))))
 
-(defconst chess-piece-name-table
-  '(("queen"  . ?q)
-    ("rook"   . ?r)
-    ("knight" . ?n)
-    ("bishop" . ?b)))
-
 (defun chess-ply-castling-changes (position &optional long king-index)
   "Create castling changes; this function supports Fischer Random castling."
   (cl-assert (vectorp position))
@@ -153,19 +154,19 @@
 	 (rook (chess-pos-can-castle position (if color
 						  (if long ?Q ?K)
 						(if long ?q ?k))))
-	 (bias (if long -1 1)) pos)
+	 (direction (if long chess-direction-west chess-direction-east)) pos)
     (when rook
-      (setq pos (chess-incr-index king 0 bias))
-      (while (and pos (not (equal pos rook))
+      (setq pos (chess-next-index king direction))
+      (while (and pos (/= pos rook)
 		  (chess-pos-piece-p position pos ? )
 		  (or (and long (< (chess-index-file pos) 2))
 		      (chess-pos-legal-candidates
 		       position color pos (list king))))
-	(setq pos (chess-incr-index pos 0 bias)))
-      (if (equal pos rook)
-	  (list king (chess-rf-to-index (if color 7 0) (if long 2 6))
-		rook (chess-rf-to-index (if color 7 0) (if long 3 5))
-		(if long :long-castle :castle))))))
+	(setq pos (chess-next-index pos direction)))
+      (when (equal pos rook)
+	(list king (if color (if long #o72 #o76) (if long #o02 #o06))
+	      rook (if color (if long #o73 #o75) (if long #o03 #o05))
+	      (if long :long-castle :castle))))))
 
 (chess-message-catalog 'english
   '((ambiguous-promotion . "Promotion without :promote keyword")))
@@ -304,7 +305,7 @@ KEYWORDS allowed are:
   :any   return t if any piece can move at all
   :color <t or nil>
   :piece <piece character>
-  :file <number 0 to 7> [can only be used if :piece is present]
+  :file <number 0 to 7> [:piece or :color must be present]
   :index <coordinate index>
   :target <specific target index>
   :candidates <list of inddices>
@@ -357,7 +358,7 @@ position object passed in."
 		 (let (candidates)
 		   (dotimes (rank 8)
 		     (setq pos (chess-rf-to-index rank file))
-		     (if (chess-pos-piece-p position pos piece)
+		     (if (chess-pos-piece-p position pos (or piece color))
 			 (push pos candidates)))
 		   candidates))
 		(t
